module Tests exposing (..)

import Dict
import Expect exposing (equal)
import Fuzz
import RemoveDiacritics exposing (lookupTable, removeDiacritics)
import String.Normalize as Normalize
import Test exposing (..)


removeDiacriticsTests : Test
removeDiacriticsTests =
    Test.describe "String.Normalize.removeDiacritics"
        [ test "empty string unchanged" <|
            \_ ->
                removeDiacritics "" |> equal ""
        , test "removes lowercase accents" <|
            \_ ->
                removeDiacritics "éeaèüàäö"
                    |> equal "eeaeuaao"
        , test "removes uppercase accents" <|
            \_ ->
                removeDiacritics "ÉEAÈÜÀÄÖ"
                    |> equal "EEAEUAAO"
        , test "removes ligatures" <|
            \_ ->
                removeDiacritics "Æƕ"
                    |> equal "AEhv"
        , test "normalizes a sentence" <|
            \_ ->
                removeDiacritics "La liberté commence où l'ignorance finit."
                    |> equal "La liberte commence ou l'ignorance finit."
        , test "don't touch punctuation" <|
            \_ ->
                removeDiacritics "é()/& abc"
                    |> equal "e()/& abc"
        , test "don't touch non latin characters" <|
            \_ ->
                removeDiacritics "こんにちは"
                    |> equal "こんにちは"
        , fuzz Fuzz.string "don't touch ASCII" <|
            \randomAscii ->
                removeDiacritics randomAscii
                    |> equal randomAscii
        , fuzz diacritics "always change diacritics" <|
            \randomWithDiacritics ->
                removeDiacritics randomWithDiacritics
                    |> (if randomWithDiacritics == "" then
                            Expect.equal ""

                        else
                            Expect.notEqual randomWithDiacritics
                       )
        , fuzz
            withDiacritics
            "second pass does nothing"
          <|
            \randomString ->
                let
                    firstPass =
                        removeDiacritics randomString

                    secondPass =
                        removeDiacritics firstPass
                in
                Expect.equal firstPass secondPass
        , fuzz
            unicode
            "same as original"
          <|
            \randomString ->
                let
                    faster =
                        removeDiacritics randomString

                    original =
                        Normalize.removeDiacritics randomString
                in
                Expect.equal faster original
        ]


unicode : Fuzz.Fuzzer String
unicode =
    Fuzz.map
        String.fromList
        (Fuzz.list (Fuzz.map Char.fromCode (Fuzz.intRange 0 0x0010FFFF)))


withDiacritics : Fuzz.Fuzzer String
withDiacritics =
    Fuzz.map
        String.fromList
        (Fuzz.list (Fuzz.oneOf [ diacritic, Fuzz.char ]))


diacritics : Fuzz.Fuzzer String
diacritics =
    Fuzz.map String.fromList (Fuzz.list diacritic)


diacritic : Fuzz.Fuzzer Char
diacritic =
    lookupTable
        |> Dict.keys
        |> List.map Fuzz.constant
        |> Fuzz.oneOf
